@@ -1,6 +1,6 @@
 /* Generic symbol-table support for the BFD library.
    Copyright 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-   2000, 2001
+   2000, 2001, 2002
    Free Software Foundation, Inc.
    Written by Cygnus Support.
 
@@ -183,131 +183,131 @@ CODE_FRAGMENT
 .
 .typedef struct symbol_cache_entry
 .{
-.	{* A pointer to the BFD which owns the symbol. This information
-.	   is necessary so that a back end can work out what additional
-.   	   information (invisible to the application writer) is carried
-.	   with the symbol.
+.  {* A pointer to the BFD which owns the symbol. This information
+.     is necessary so that a back end can work out what additional
+.     information (invisible to the application writer) is carried
+.     with the symbol.
 .
-.	   This field is *almost* redundant, since you can use section->owner
-.	   instead, except that some symbols point to the global sections
-.	   bfd_{abs,com,und}_section.  This could be fixed by making
-.	   these globals be per-bfd (or per-target-flavor).  FIXME. *}
+.     This field is *almost* redundant, since you can use section->owner
+.     instead, except that some symbols point to the global sections
+.     bfd_{abs,com,und}_section.  This could be fixed by making
+.     these globals be per-bfd (or per-target-flavor).  FIXME.  *}
+.  struct _bfd *the_bfd; {* Use bfd_asymbol_bfd(sym) to access this field.  *}
 .
-.  struct _bfd *the_bfd; {* Use bfd_asymbol_bfd(sym) to access this field. *}
+.  {* The text of the symbol. The name is left alone, and not copied; the
+.     application may not alter it.  *}
+.  const char *name;
 .
-.	{* The text of the symbol. The name is left alone, and not copied; the
-.	   application may not alter it. *}
-.  CONST char *name;
-.
-.	{* The value of the symbol.  This really should be a union of a
-.          numeric value with a pointer, since some flags indicate that
-.          a pointer to another symbol is stored here.  *}
+.  {* The value of the symbol.  This really should be a union of a
+.     numeric value with a pointer, since some flags indicate that
+.     a pointer to another symbol is stored here.  *}
 .  symvalue value;
 .
-.	{* Attributes of a symbol: *}
-.
+.  {* Attributes of a symbol.  *}
 .#define BSF_NO_FLAGS    0x00
 .
-.	{* The symbol has local scope; <<static>> in <<C>>. The value
-. 	   is the offset into the section of the data. *}
+.  {* The symbol has local scope; <<static>> in <<C>>. The value
+.     is the offset into the section of the data.  *}
 .#define BSF_LOCAL	0x01
 .
-.	{* The symbol has global scope; initialized data in <<C>>. The
-.	   value is the offset into the section of the data. *}
+.  {* The symbol has global scope; initialized data in <<C>>. The
+.     value is the offset into the section of the data.  *}
 .#define BSF_GLOBAL	0x02
 .
-.	{* The symbol has global scope and is exported. The value is
-.	   the offset into the section of the data. *}
-.#define BSF_EXPORT	BSF_GLOBAL {* no real difference *}
+.  {* The symbol has global scope and is exported. The value is
+.     the offset into the section of the data.  *}
+.#define BSF_EXPORT	BSF_GLOBAL {* No real difference.  *}
 .
-.	{* A normal C symbol would be one of:
-.	   <<BSF_LOCAL>>, <<BSF_FORT_COMM>>,  <<BSF_UNDEFINED>> or
-.	   <<BSF_GLOBAL>> *}
+.  {* A normal C symbol would be one of:
+.     <<BSF_LOCAL>>, <<BSF_FORT_COMM>>,  <<BSF_UNDEFINED>> or
+.     <<BSF_GLOBAL>>.  *}
 .
-.	{* The symbol is a debugging record. The value has an arbitary
-.	   meaning, unless BSF_DEBUGGING_RELOC is also set.  *}
+.  {* The symbol is a debugging record. The value has an arbitary
+.     meaning, unless BSF_DEBUGGING_RELOC is also set.  *}
 .#define BSF_DEBUGGING	0x08
 .
-.	{* The symbol denotes a function entry point.  Used in ELF,
-.	   perhaps others someday.  *}
+.  {* The symbol denotes a function entry point.  Used in ELF,
+.     perhaps others someday.  *}
 .#define BSF_FUNCTION    0x10
 .
-.	{* Used by the linker. *}
+.  {* Used by the linker.  *}
 .#define BSF_KEEP        0x20
 .#define BSF_KEEP_G      0x40
 .
-.	{* A weak global symbol, overridable without warnings by
-.	   a regular global symbol of the same name.  *}
+.  {* A weak global symbol, overridable without warnings by
+.     a regular global symbol of the same name.  *}
 .#define BSF_WEAK        0x80
 .
-.       {* This symbol was created to point to a section, e.g. ELF's
-.	   STT_SECTION symbols.  *}
+.  {* This symbol was created to point to a section, e.g. ELF's
+.     STT_SECTION symbols.  *}
 .#define BSF_SECTION_SYM 0x100
 .
-.	{* The symbol used to be a common symbol, but now it is
-.	   allocated. *}
+.  {* The symbol used to be a common symbol, but now it is
+.     allocated.  *}
 .#define BSF_OLD_COMMON  0x200
 .
-.	{* The default value for common data. *}
+.  {* The default value for common data.  *}
 .#define BFD_FORT_COMM_DEFAULT_VALUE 0
 .
-.	{* In some files the type of a symbol sometimes alters its
-.	   location in an output file - ie in coff a <<ISFCN>> symbol
-.	   which is also <<C_EXT>> symbol appears where it was
-.	   declared and not at the end of a section.  This bit is set
-.  	   by the target BFD part to convey this information. *}
-.
+.  {* In some files the type of a symbol sometimes alters its
+.     location in an output file - ie in coff a <<ISFCN>> symbol
+.     which is also <<C_EXT>> symbol appears where it was
+.     declared and not at the end of a section.  This bit is set
+.     by the target BFD part to convey this information.  *}
 .#define BSF_NOT_AT_END    0x400
 .
-.	{* Signal that the symbol is the label of constructor section. *}
+.  {* Signal that the symbol is the label of constructor section.  *}
 .#define BSF_CONSTRUCTOR   0x800
 .
-.	{* Signal that the symbol is a warning symbol.  The name is a
-.	   warning.  The name of the next symbol is the one to warn about;
-.	   if a reference is made to a symbol with the same name as the next
-.	   symbol, a warning is issued by the linker. *}
+.  {* Signal that the symbol is a warning symbol.  The name is a
+.     warning.  The name of the next symbol is the one to warn about;
+.     if a reference is made to a symbol with the same name as the next
+.     symbol, a warning is issued by the linker.  *}
 .#define BSF_WARNING       0x1000
 .
-.	{* Signal that the symbol is indirect.  This symbol is an indirect
-.	   pointer to the symbol with the same name as the next symbol. *}
+.  {* Signal that the symbol is indirect.  This symbol is an indirect
+.     pointer to the symbol with the same name as the next symbol.  *}
 .#define BSF_INDIRECT      0x2000
 .
-.	{* BSF_FILE marks symbols that contain a file name.  This is used
-.	   for ELF STT_FILE symbols.  *}
+.  {* BSF_FILE marks symbols that contain a file name.  This is used
+.     for ELF STT_FILE symbols.  *}
 .#define BSF_FILE          0x4000
 .
-.	{* Symbol is from dynamic linking information.  *}
+.  {* Symbol is from dynamic linking information.  *}
 .#define BSF_DYNAMIC	   0x8000
 .
-.       {* The symbol denotes a data object.  Used in ELF, and perhaps
-.          others someday.  *}
+.  {* The symbol denotes a data object.  Used in ELF, and perhaps
+.     others someday.  *}
 .#define BSF_OBJECT	   0x10000
 .
-.       {* This symbol is a debugging symbol.  The value is the offset
-.          into the section of the data.  BSF_DEBUGGING should be set
-.          as well.  *}
+.  {* This symbol is a debugging symbol.  The value is the offset
+.     into the section of the data.  BSF_DEBUGGING should be set
+.     as well.  *}
 .#define BSF_DEBUGGING_RELOC 0x20000
 .
 .  flagword flags;
 .
-.	{* A pointer to the section to which this symbol is
-.	   relative.  This will always be non NULL, there are special
-.          sections for undefined and absolute symbols.  *}
+.  {* A pointer to the section to which this symbol is
+.     relative.  This will always be non NULL, there are special
+.     sections for undefined and absolute symbols.  *}
 .  struct sec *section;
 .
-.	{* Back end special data.  *}
+.  {* Back end special data.  *}
 .  union
 .    {
 .      PTR p;
 .      bfd_vma i;
-.    } udata;
+.    }
+.  udata;
+.}
+.asymbol;
 .
-.} asymbol;
 */
 
 #include "bfd.h"
 #include "sysdep.h"
 #include "libbfd.h"
+#include "safe-ctype.h"
 #include "bfdlink.h"
 #include "aout/stab_gnu.h"
 
@@ -334,7 +334,7 @@ DESCRIPTION
 
 .#define bfd_get_symtab_upper_bound(abfd) \
 .     BFD_SEND (abfd, _bfd_get_symtab_upper_bound, (abfd))
-
+.
 */
 
 /*
@@ -379,6 +379,7 @@ DESCRIPTION
 
 .#define bfd_is_local_label_name(abfd, name) \
 .     BFD_SEND (abfd, _bfd_is_local_label_name, (abfd, name))
+.
 */
 
 /*
@@ -395,7 +396,7 @@ DESCRIPTION
 .#define bfd_canonicalize_symtab(abfd, location) \
 .     BFD_SEND (abfd, _bfd_canonicalize_symtab,\
 .                  (abfd, location))
-
+.
 */
 
 /*
@@ -491,8 +492,34 @@ DESCRIPTION
 
 .#define bfd_make_empty_symbol(abfd) \
 .     BFD_SEND (abfd, _bfd_make_empty_symbol, (abfd))
+.
 */
 
+/*
+FUNCTION
+	_bfd_generic_make_empty_symbol
+
+SYNOPSIS
+	asymbol *_bfd_generic_make_empty_symbol (bfd *);
+
+DESCRIPTION
+	Create a new <<asymbol>> structure for the BFD @var{abfd}
+	and return a pointer to it.  Used by core file routines,
+	binary back-end and anywhere else where no private info
+	is needed.
+*/
+
+asymbol *
+_bfd_generic_make_empty_symbol (abfd)
+     bfd *abfd;
+{
+  bfd_size_type amt = sizeof (asymbol);
+  asymbol *new = (asymbol *) bfd_zalloc (abfd, amt);
+  if (new)
+    new->the_bfd = abfd;
+  return new;
+}
+
 /*
 FUNCTION
 	bfd_make_debug_symbol
@@ -504,36 +531,39 @@ DESCRIPTION
 
 .#define bfd_make_debug_symbol(abfd,ptr,size) \
 .        BFD_SEND (abfd, _bfd_make_debug_symbol, (abfd, ptr, size))
+.
 */
 
 struct section_to_type
 {
-  CONST char *section;
+  const char *section;
   char type;
 };
 
 /* Map section names to POSIX/BSD single-character symbol types.
    This table is probably incomplete.  It is sorted for convenience of
    adding entries.  Since it is so short, a linear search is used.  */
-static CONST struct section_to_type stt[] =
+static const struct section_to_type stt[] =
 {
-  {"*DEBUG*", 'N'},
   {".bss", 'b'},
-  {"zerovars", 'b'},		/* MRI .bss */
+  {"code", 't'},		/* MRI .text */
   {".data", 'd'},
-  {"vars", 'd'},		/* MRI .data */
+  {"*DEBUG*", 'N'},
+  {".debug", 'N'},              /* MSVC's .debug (non-standard debug syms) */
+  {".drectve", 'i'},            /* MSVC's .drective section */
+  {".edata", 'e'},              /* MSVC's .edata (export) section */
+  {".fini", 't'},		/* ELF fini section */
+  {".idata", 'i'},              /* MSVC's .idata (import) section */
+  {".init", 't'},		/* ELF init section */
+  {".pdata", 'p'},              /* MSVC's .pdata (stack unwind) section */
   {".rdata", 'r'},		/* Read only data.  */
   {".rodata", 'r'},		/* Read only data.  */
   {".sbss", 's'},		/* Small BSS (uninitialized data).  */
   {".scommon", 'c'},		/* Small common.  */
   {".sdata", 'g'},		/* Small initialized data.  */
   {".text", 't'},
-  {"code", 't'},		/* MRI .text */
-  {".drectve", 'i'},            /* MSVC's .drective section */
-  {".idata", 'i'},              /* MSVC's .idata (import) section */
-  {".edata", 'e'},              /* MSVC's .edata (export) section */
-  {".pdata", 'p'},              /* MSVC's .pdata (stack unwind) section */
-  {".debug", 'N'},              /* MSVC's .debug (non-standard debug syms) */
+  {"vars", 'd'},		/* MRI .data */
+  {"zerovars", 'b'},		/* MRI .bss */
   {0, 0}
 };
 
@@ -547,7 +577,7 @@ static char
 coff_section_type (s)
      const char *s;
 {
-  CONST struct section_to_type *t;
+  const struct section_to_type *t;
 
   for (t = &stt[0]; t->section; t++)
     if (!strncmp (s, t->section, strlen (t->section)))
@@ -556,13 +586,6 @@ coff_section_type (s)
   return '?';
 }
 
-#ifndef islower
-#define islower(c) ((c) >= 'a' && (c) <= 'z')
-#endif
-#ifndef toupper
-#define toupper(c) (islower(c) ? ((c) & ~0x20) : (c))
-#endif
-
 /*
 FUNCTION
 	bfd_decode_symclass
@@ -617,7 +640,7 @@ bfd_decode_symclass (symbol)
   else
     return '?';
   if (symbol->flags & BSF_GLOBAL)
-    c = toupper (c);
+    c = TOUPPER (c);
   return c;
 
   /* We don't have to handle these cases just yet, but we will soon:
@@ -697,7 +720,7 @@ DESCRIPTION
 .#define bfd_copy_private_symbol_data(ibfd, isymbol, obfd, osymbol) \
 .     BFD_SEND (obfd, _bfd_copy_private_symbol_data, \
 .		(ibfd, isymbol, obfd, osymbol))
-
+.
 */
 
 /* The generic version of the function which returns mini symbols.
@@ -721,8 +744,10 @@ _bfd_generic_read_minisymbols (abfd, dynamic, minisymsp, sizep)
     storage = bfd_get_symtab_upper_bound (abfd);
   if (storage < 0)
     goto error_return;
+  if (storage == 0)
+    return 0;
 
-  syms = (asymbol **) bfd_malloc ((size_t) storage);
+  syms = (asymbol **) bfd_malloc ((bfd_size_type) storage);
   if (syms == NULL)
     goto error_return;
 
@@ -852,7 +877,8 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
   bfd_byte *last_stab = NULL;
   bfd_size_type stroff;
   struct indexentry *indexentry;
-  char *directory_name, *file_name;
+  char *file_name;
+  char *directory_name;
   int saw_fun;
 
   *pfound = false;
@@ -898,11 +924,10 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
       arelent **reloc_vector;
       int i;
       char *name;
-      char *file_name;
-      char *directory_name;
       char *function_name;
+      bfd_size_type amt = sizeof *info;
 
-      info = (struct stab_find_info *) bfd_zalloc (abfd, sizeof *info);
+      info = (struct stab_find_info *) bfd_zalloc (abfd, amt);
       if (info == NULL)
 	return false;
 
@@ -929,10 +954,10 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
       if (info->stabs == NULL || info->strs == NULL)
 	return false;
 
-      if (! bfd_get_section_contents (abfd, info->stabsec, info->stabs, 0,
-				      stabsize)
-	  || ! bfd_get_section_contents (abfd, info->strsec, info->strs, 0,
-					 strsize))
+      if (! bfd_get_section_contents (abfd, info->stabsec, info->stabs,
+				      (bfd_vma) 0, stabsize)
+	  || ! bfd_get_section_contents (abfd, info->strsec, info->strs,
+					 (bfd_vma) 0, strsize))
 	return false;
 
       /* If this is a relocateable object file, we have to relocate
@@ -942,7 +967,7 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
       reloc_size = bfd_get_reloc_upper_bound (abfd, info->stabsec);
       if (reloc_size < 0)
 	return false;
-      reloc_vector = (arelent **) bfd_malloc (reloc_size);
+      reloc_vector = (arelent **) bfd_malloc ((bfd_size_type) reloc_size);
       if (reloc_vector == NULL && reloc_size != 0)
 	return false;
       reloc_count = bfd_canonicalize_reloc (abfd, info->stabsec, reloc_vector,
@@ -983,7 +1008,7 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 	      val &= r->howto->src_mask;
 	      sym = *r->sym_ptr_ptr;
 	      val += sym->value + sym->section->vma + r->addend;
-	      bfd_put_32 (abfd, val, info->stabs + r->address);
+	      bfd_put_32 (abfd, (bfd_vma) val, info->stabs + r->address);
 	    }
 	}
 
@@ -1033,10 +1058,9 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 	return true;
       ++info->indextablesize;
 
-      info->indextable = ((struct indexentry *)
-			  bfd_alloc (abfd,
-				     (sizeof (struct indexentry)
-				      * info->indextablesize)));
+      amt = info->indextablesize;
+      amt *= sizeof (struct indexentry);
+      info->indextable = (struct indexentry *) bfd_alloc (abfd, amt);
       if (info->indextable == NULL)
 	return false;
 
@@ -1154,7 +1178,8 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
       ++i;
 
       info->indextablesize = i;
-      qsort (info->indextable, i, sizeof (struct indexentry), cmpindexentry);
+      qsort (info->indextable, (size_t) i, sizeof (struct indexentry),
+	     cmpindexentry);
 
       *pinfo = (PTR) info;
     }
@@ -1276,9 +1301,8 @@ _bfd_stab_section_find_nearest_line (abfd, symbols, section, offset, pfound,
 	{
 	  if (info->filename != NULL)
 	    free (info->filename);
-	  info->filename = (char *) bfd_malloc (dirlen +
-						strlen (file_name)
-						+ 1);
+	  info->filename = (char *) bfd_malloc ((bfd_size_type) dirlen
+						+ strlen (file_name) + 1);
 	  if (info->filename == NULL)
 	    return false;
 	  strcpy (info->filename, directory_name);
