@@ -2,5 +2,4 @@ TEMPLATE_NAME=aix
 SCRIPT_NAME=aix
 OUTPUT_FORMAT="aixcoff-rs6000"
 ARCH=rs6000
-SYSCALL_MASK=0x77
-SYMBOL_MODE_MASK=0x0d
+
