@@ -1,6 +1,6 @@
 /* This module handles expression trees.
    Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
-   2001
+   2001, 2002
    Free Software Foundation, Inc.
    Written by Steve Chamberlain of Cygnus Support <sac@cygnus.com>.
 
@@ -38,6 +38,8 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "ldexp.h"
 #include "ldgram.h"
 #include "ldlang.h"
+#include "libiberty.h"
+#include "safe-ctype.h"
 
 static void exp_print_token PARAMS ((token_code_type code));
 static void make_abs PARAMS ((etree_value_type *ptr));
@@ -45,7 +47,7 @@ static etree_value_type new_abs PARAMS ((bfd_vma value));
 static void check PARAMS ((lang_output_section_statement_type *os,
 			   const char *name, const char *op));
 static etree_value_type new_rel
-  PARAMS ((bfd_vma value, lang_output_section_statement_type *section));
+  PARAMS ((bfd_vma, char *, lang_output_section_statement_type *section));
 static etree_value_type new_rel_from_section
   PARAMS ((bfd_vma value, lang_output_section_statement_type *section));
 static etree_value_type fold_binary
@@ -63,16 +65,20 @@ static etree_value_type exp_fold_tree_no_dot
 	   lang_output_section_statement_type *current_section,
 	   lang_phase_type allocation_done));
 
+struct exp_data_seg exp_data_seg;
+
 static void
 exp_print_token (code)
      token_code_type code;
 {
-  static CONST struct {
+  static CONST struct
+  {
     token_code_type code;
-    char *name;
-  } table[] = {
+    char * name;
+  }
+  table[] =
+  {
     { INT, "int" },
-    { REL, "relocateable" },
     { NAME, "NAME" },
     { PLUSEQ, "+=" },
     { MINUSEQ, "-=" },
@@ -89,40 +95,47 @@ exp_print_token (code)
     { LE, "<=" },
     { GE, ">=" },
     { LSHIFT, "<<" },
-    { RSHIFT, ">>=" },
+    { RSHIFT, ">>" },
     { ALIGN_K, "ALIGN" },
     { BLOCK, "BLOCK" },
-    { SECTIONS, "SECTIONS" },
-    { SIZEOF_HEADERS, "SIZEOF_HEADERS" },
-    { NEXT, "NEXT" },
-    { SIZEOF, "SIZEOF" },
-    { ADDR, "ADDR" },
-    { LOADADDR, "LOADADDR" },
-    { MEMORY, "MEMORY" },
-    { DEFINED, "DEFINED" },
-    { TARGET_K, "TARGET" },
-    { SEARCH_DIR, "SEARCH_DIR" },
-    { MAP, "MAP" },
     { QUAD, "QUAD" },
     { SQUAD, "SQUAD" },
     { LONG, "LONG" },
     { SHORT, "SHORT" },
     { BYTE, "BYTE" },
+    { SECTIONS, "SECTIONS" },
+    { SIZEOF_HEADERS, "SIZEOF_HEADERS" },
+    { MEMORY, "MEMORY" },
+    { DEFINED, "DEFINED" },
+    { TARGET_K, "TARGET" },
+    { SEARCH_DIR, "SEARCH_DIR" },
+    { MAP, "MAP" },
     { ENTRY, "ENTRY" },
-    { 0, (char *) NULL }
+    { NEXT, "NEXT" },
+    { SIZEOF, "SIZEOF" },
+    { ADDR, "ADDR" },
+    { LOADADDR, "LOADADDR" },
+    { MAX_K, "MAX_K" },
+    { REL, "relocateable" },
+    { DATA_SEGMENT_ALIGN, "DATA_SEGMENT_ALIGN" },
+    { DATA_SEGMENT_END, "DATA_SEGMENT_END" }
   };
   unsigned int idx;
 
-  for (idx = 0; table[idx].name != (char *) NULL; idx++)
+  for (idx = ARRAY_SIZE (table); idx--;)
     {
       if (table[idx].code == code)
 	{
-	  fprintf (config.map_file, "%s", table[idx].name);
+	  fprintf (config.map_file, " %s ", table[idx].name);
 	  return;
 	}
     }
-  /* Not in table, just print it alone */
-  fprintf (config.map_file, "%c", code);
+
+  /* Not in table, just print it alone.  */
+  if (code < 127)
+    fprintf (config.map_file, " %c ", code);
+  else
+    fprintf (config.map_file, " <code %d> ", code);
 }
 
 static void
@@ -164,9 +177,22 @@ exp_intop (value)
   etree_type *new = (etree_type *) stat_alloc (sizeof (new->value));
   new->type.node_code = INT;
   new->value.value = value;
+  new->value.str = NULL;
   new->type.node_class = etree_value;
   return new;
+}
 
+etree_type *
+exp_bigintop (value, str)
+     bfd_vma value;
+     char *str;
+{
+  etree_type *new = (etree_type *) stat_alloc (sizeof (new->value));
+  new->type.node_code = INT;
+  new->value.value = value;
+  new->value.str = str;
+  new->type.node_class = etree_value;
+  return new;
 }
 
 /* Build an expression representing an unnamed relocateable value.  */
@@ -185,13 +211,15 @@ exp_relop (section, value)
 }
 
 static etree_value_type
-new_rel (value, section)
+new_rel (value, str, section)
      bfd_vma value;
+     char *str;
      lang_output_section_statement_type *section;
 {
   etree_value_type new;
   new.valid_p = true;
   new.value = value;
+  new.str = str;
   new.section = section;
   return new;
 }
@@ -204,6 +232,7 @@ new_rel_from_section (value, section)
   etree_value_type new;
   new.valid_p = true;
   new.value = value;
+  new.str = NULL;
   new.section = section;
 
   new.value -= section->bfd_section->vma;
@@ -306,6 +335,33 @@ fold_binary (tree, current_section, allocation_done, dot, dotp)
 		result = other;
 	      break;
 
+	    case DATA_SEGMENT_ALIGN:
+	      if (allocation_done != lang_first_phase_enum
+		  && current_section == abs_output_section
+		  && (exp_data_seg.phase == exp_dataseg_none
+		      || exp_data_seg.phase == exp_dataseg_adjust
+		      || allocation_done != lang_allocating_phase_enum))
+		{
+		  bfd_vma maxpage = result.value;
+
+		  result.value = ALIGN_N (dot, maxpage);
+		  if (exp_data_seg.phase != exp_dataseg_adjust)
+		    {
+		      result.value += dot & (maxpage - 1);
+		      if (allocation_done == lang_allocating_phase_enum)
+			{
+			  exp_data_seg.phase = exp_dataseg_align_seen;
+			  exp_data_seg.base = result.value;
+			  exp_data_seg.pagesize = other.value;
+			}
+		    }
+		  else if (other.value < maxpage)
+		    result.value += dot & (maxpage - other.value);
+		}
+	      else
+		result.valid_p = false;
+	      break;
+
 	    default:
 	      FAIL ();
 	    }
@@ -335,6 +391,7 @@ fold_name (tree, current_section, allocation_done, dot)
      bfd_vma dot;
 {
   etree_value_type result;
+
   switch (tree->type.node_code)
     {
     case SIZEOF_HEADERS:
@@ -410,6 +467,7 @@ fold_name (tree, current_section, allocation_done, dot)
 			 being linked with -R?  */
 		      result = new_rel ((h->u.def.value
 					 + h->u.def.section->output_offset),
+					NULL,
 					os);
 		    }
 		}
@@ -427,7 +485,7 @@ fold_name (tree, current_section, allocation_done, dot)
 
 	  os = lang_output_section_find (tree->name.name);
 	  check (os, tree->name.name, "ADDR");
-	  result = new_rel (0, os);
+	  result = new_rel (0, NULL, os);
 	}
       else
 	result = invalid ();
@@ -441,7 +499,7 @@ fold_name (tree, current_section, allocation_done, dot)
 	  os = lang_output_section_find (tree->name.name);
 	  check (os, tree->name.name, "LOADADDR");
 	  if (os->load_base == NULL)
-	    result = new_rel (0, os);
+	    result = new_rel (0, NULL, os);
 	  else
 	    result = exp_fold_tree_no_dot (os->load_base,
 					   abs_output_section,
@@ -492,7 +550,7 @@ exp_fold_tree (tree, current_section, allocation_done, dot, dotp)
   switch (tree->type.node_class)
     {
     case etree_value:
-      result = new_rel (tree->value.value, current_section);
+      result = new_rel (tree->value.value, tree->value.str, current_section);
       break;
 
     case etree_rel:
@@ -502,6 +560,7 @@ exp_fold_tree (tree, current_section, allocation_done, dot, dotp)
 	result = new_rel ((tree->rel.value
 			   + tree->rel.section->output_section->vma
 			   + tree->rel.section->output_offset),
+			  NULL,
 			  current_section);
       break;
 
@@ -569,6 +628,23 @@ exp_fold_tree (tree, current_section, allocation_done, dot, dotp)
 		result.valid_p = false;
 	      break;
 
+	    case DATA_SEGMENT_END:
+	      if (allocation_done != lang_first_phase_enum
+		  && current_section == abs_output_section
+		  && (exp_data_seg.phase == exp_dataseg_align_seen
+		      || exp_data_seg.phase == exp_dataseg_adjust
+		      || allocation_done != lang_allocating_phase_enum))
+		{
+		  if (exp_data_seg.phase == exp_dataseg_align_seen)
+		    {
+		      exp_data_seg.phase = exp_dataseg_end_seen;
+		      exp_data_seg.end = result.value;
+		    }
+		}
+	      else
+		result.valid_p = false;
+	      break;
+
 	    default:
 	      FAIL ();
 	      break;
@@ -597,7 +673,7 @@ exp_fold_tree (tree, current_section, allocation_done, dot, dotp)
     case etree_provided:
       if (tree->assign.dst[0] == '.' && tree->assign.dst[1] == 0)
 	{
-	  /* Assignment to dot can only be done during allocation */
+	  /* Assignment to dot can only be done during allocation.  */
 	  if (tree->type.node_class != etree_assign)
 	    einfo (_("%F%S can not PROVIDE assignment to location counter\n"));
 	  if (allocation_done == lang_allocating_phase_enum
@@ -606,7 +682,7 @@ exp_fold_tree (tree, current_section, allocation_done, dot, dotp)
 	    {
 	      result = exp_fold_tree (tree->assign.src,
 				      current_section,
-				      lang_allocating_phase_enum, dot,
+				      allocation_done, dot,
 				      dotp);
 	      if (! result.valid_p)
 		einfo (_("%F%S invalid assignment to location counter\n"));
@@ -622,10 +698,8 @@ exp_fold_tree (tree, current_section, allocation_done, dot, dotp)
 				 + current_section->bfd_section->vma);
 		      if (nextdot < dot
 			  && current_section != abs_output_section)
-			{
-			  einfo (_("%F%S cannot move location counter backwards (from %V to %V)\n"),
-				 dot, nextdot);
-			}
+			einfo (_("%F%S cannot move location counter backwards (from %V to %V)\n"),
+			       dot, nextdot);
 		      else
 			*dotp = nextdot;
 		    }
@@ -740,9 +814,8 @@ exp_trinop (code, cond, lhs, rhs)
 			    (lang_output_section_statement_type *) NULL,
 			    lang_first_phase_enum);
   if (r.valid_p)
-    {
-      return exp_intop (r.value);
-    }
+    return exp_intop (r.value);
+
   new = (etree_type *) stat_alloc (sizeof (new->trinary));
   memcpy ((char *) new, (char *) &value, sizeof (new->trinary));
   return new;
@@ -762,9 +835,8 @@ exp_unop (code, child)
   r = exp_fold_tree_no_dot (&value, abs_output_section,
 			    lang_first_phase_enum);
   if (r.valid_p)
-    {
-      return exp_intop (r.value);
-    }
+    return exp_intop (r.value);
+
   new = (etree_type *) stat_alloc (sizeof (new->unary));
   memcpy ((char *) new, (char *) &value, sizeof (new->unary));
   return new;
@@ -785,9 +857,8 @@ exp_nameop (code, name)
 			    (lang_output_section_statement_type *) NULL,
 			    lang_first_phase_enum);
   if (r.valid_p)
-    {
-      return exp_intop (r.value);
-    }
+    return exp_intop (r.value);
+
   new = (etree_type *) stat_alloc (sizeof (new->name));
   memcpy ((char *) new, (char *) &value, sizeof (new->name));
   return new;
@@ -810,9 +881,7 @@ exp_assop (code, dst, src)
 
 #if 0
   if (exp_fold_tree_no_dot (&value, &result))
-    {
-      return exp_intop (result);
-    }
+    return exp_intop (result);
 #endif
   new = (etree_type *) stat_alloc (sizeof (new->assign));
   memcpy ((char *) new, (char *) &value, sizeof (new->assign));
@@ -857,6 +926,15 @@ void
 exp_print_tree (tree)
      etree_type *tree;
 {
+  if (config.map_file == NULL)
+    config.map_file = stderr;
+  
+  if (tree == NULL)
+    {
+      minfo ("NULL TREE\n");
+      return;
+    }
+  
   switch (tree->type.node_class)
     {
     case etree_value:
@@ -870,14 +948,10 @@ exp_print_tree (tree)
     case etree_assign:
 #if 0
       if (tree->assign.dst->sdefs != (asymbol *) NULL)
-	{
-	  fprintf (config.map_file, "%s (%x) ", tree->assign.dst->name,
-		   tree->assign.dst->sdefs->value);
-	}
+	fprintf (config.map_file, "%s (%x) ", tree->assign.dst->name,
+		 tree->assign.dst->sdefs->value);
       else
-	{
-	  fprintf (config.map_file, "%s (UNDEFINED)", tree->assign.dst->name);
-	}
+	fprintf (config.map_file, "%s (UNDEFINED)", tree->assign.dst->name);
 #endif
       fprintf (config.map_file, "%s", tree->assign.dst);
       exp_print_token (tree->type.node_code);
@@ -970,6 +1044,65 @@ exp_get_value_int (tree, def, name, allocation_done)
   return (int) exp_get_vma (tree, (bfd_vma) def, name, allocation_done);
 }
 
+fill_type *
+exp_get_fill (tree, def, name, allocation_done)
+     etree_type *tree;
+     fill_type *def;
+     char *name;
+     lang_phase_type allocation_done;
+{
+  fill_type *fill;
+  etree_value_type r;
+  size_t len;
+  unsigned int val;
+
+  if (tree == NULL)
+    return def;
+
+  r = exp_fold_tree_no_dot (tree, abs_output_section, allocation_done);
+  if (! r.valid_p && name != NULL)
+    einfo (_("%F%S nonconstant expression for %s\n"), name);
+
+  if (r.str != NULL && (len = strlen (r.str)) != 0)
+    {
+      unsigned char *dst;
+      unsigned char *s;
+      fill = (fill_type *) xmalloc ((len + 1) / 2 + sizeof (*fill) - 1);
+      fill->size = (len + 1) / 2;
+      dst = fill->data;
+      s = r.str;
+      val = 0;
+      do
+	{
+	  unsigned int digit;
+
+	  digit = *s++ - '0';
+	  if (digit > 9)
+	    digit = (digit - 'A' + '0' + 10) & 0xf;
+	  val <<= 4;
+	  val += digit;
+	  --len;
+	  if ((len & 1) == 0)
+	    {
+	      *dst++ = val;
+	      val = 0;
+	    }
+	}
+      while (len != 0);
+    }
+  else
+    {
+      fill = (fill_type *) xmalloc (4 + sizeof (*fill) - 1);
+      val = r.value;
+      fill->data[0] = (val >> 24) & 0xff;
+      fill->data[1] = (val >> 16) & 0xff;
+      fill->data[2] = (val >>  8) & 0xff;
+      fill->data[3] = (val >>  0) & 0xff;
+      fill->size = 4;
+    }
+  return fill;
+}
+
 bfd_vma
 exp_get_abs_int (tree, def, name, allocation_done)
      etree_type *tree;
@@ -981,12 +1114,9 @@ exp_get_abs_int (tree, def, name, allocation_done)
   res = exp_fold_tree_no_dot (tree, abs_output_section, allocation_done);
 
   if (res.valid_p)
-    {
-      res.value += res.section->bfd_section->vma;
-    }
+    res.value += res.section->bfd_section->vma;
   else
-    {
-      einfo (_("%F%S non constant expression for %s\n"), name);
-    }
+    einfo (_("%F%S non constant expression for %s\n"), name);
+
   return res.value;
 }
